/*
    QuickMSG
    Copyright (C) 2014  Jeroen Vreeken <jeroen@vreeken.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#define _GNU_SOURCE
#include "settings.h"
#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <glib.h>
#include <string.h>

static bool loaded = false;

static char *settings_dir;
static char *settings_db_filename;
static char *settings_conf_filename;
static char *settings_attachment_dirname;

static char *settings_name = "me";
static char *settings_address = "foo@example.com";
static char *settings_gpg = "foosgpgsecret";
static char *settings_imap_host = "imap.example.com";
static char *settings_imap_user = "imapuser";
static char *settings_imap_pass = "imapsecret";
static char *settings_smtp_host = "smtp.example.com";
static char *settings_smtp_user = "smtpuser";
static char *settings_smtp_pass = "smtpsecret";
unsigned short settings_imap_port = 993;
unsigned short settings_smtp_port = 587;

bool load_settings(void)
{
	char *dir;
	GKeyFile *keyfile;
	
	if (loaded)
		return true;
	
	dir = getenv("HOME");
	if (!dir)
		return false;
	
	asprintf(&settings_dir, "%s/.quickmsg", dir);
	mkdir(dir, 0700);
	
	asprintf(&settings_db_filename, "%s/quickmsg.db", settings_dir);
	asprintf(&settings_conf_filename, "%s/quickmsg.conf", settings_dir);
	asprintf(&settings_attachment_dirname, "%s/quickmsg_attachments/", dir);
	mkdir(settings_attachment_dirname, 0700);

	keyfile = g_key_file_new();

	loaded = g_key_file_load_from_file(
	    keyfile, 
	    settings_conf_filename,
	    G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS,
	    NULL);
	
	if (loaded) {
		gsize groups_nr;
		gchar **groups;
		int i;
		
		groups = g_key_file_get_groups(keyfile, &groups_nr);
		
		for (i = 0; i < groups_nr; i++) {
			if (!strcmp(groups[i], "me")) {
				if (g_key_file_has_key(keyfile, groups[i], "name", NULL)) {
					settings_name = strdup(
					    g_key_file_get_string(keyfile, groups[i], "name", NULL));
				}
				if (g_key_file_has_key(keyfile, groups[i], "address", NULL)) {
					settings_address = strdup(
					    g_key_file_get_string(keyfile, groups[i], "address", NULL));
				}
				if (g_key_file_has_key(keyfile, groups[i], "pass", NULL)) {
					settings_gpg = strdup(
					    g_key_file_get_string(keyfile, groups[i], "pass", NULL));
				}
			}
			if (!strcmp(groups[i], "imap")) {
				if (g_key_file_has_key(keyfile, groups[i], "host", NULL)) {
					settings_imap_host = strdup(
					    g_key_file_get_string(keyfile, groups[i], "host", NULL));
				}
				if (g_key_file_has_key(keyfile, groups[i], "user", NULL)) {
					settings_imap_user = strdup(
					    g_key_file_get_string(keyfile, groups[i], "user", NULL));
				}
				if (g_key_file_has_key(keyfile, groups[i], "pass", NULL)) {
					settings_imap_pass = strdup(
					    g_key_file_get_string(keyfile, groups[i], "pass", NULL));
				}
				if (g_key_file_has_key(keyfile, groups[i], "port", NULL)) {
					settings_imap_port =
					    g_key_file_get_integer(keyfile, groups[i], "port", NULL);
				}
			}
			if (!strcmp(groups[i], "smtp")) {
				if (g_key_file_has_key(keyfile, groups[i], "host", NULL)) {
					settings_smtp_host = strdup(
					    g_key_file_get_string(keyfile, groups[i], "host", NULL));
				}
				if (g_key_file_has_key(keyfile, groups[i], "user", NULL)) {
					settings_smtp_user = strdup(
					    g_key_file_get_string(keyfile, groups[i], "user", NULL));
				}
				if (g_key_file_has_key(keyfile, groups[i], "pass", NULL)) {
					settings_smtp_pass = strdup(
					    g_key_file_get_string(keyfile, groups[i], "pass", NULL));
				}
				if (g_key_file_has_key(keyfile, groups[i], "port", NULL)) {
					settings_smtp_port =
					    g_key_file_get_integer(keyfile, groups[i], "port", NULL);
				}
			}
		}
		g_strfreev(groups);
		g_key_file_free(keyfile);
	}

	return true;
}

char *settings_gpg_pass_get(void)
{
	if (load_settings()) {
		return settings_gpg;
	} else {
		return NULL;
	}
}

char *settings_address_get(void)
{
	if (load_settings()) {
		return settings_address;
	} else {
		return NULL;
	}
}

char *settings_name_get(void)
{
	if (load_settings()) {
		return settings_name;
	} else {
		return NULL;
	}
}

char *settings_db_filename_get(void)
{
	if (load_settings()) {
		return settings_db_filename;
	} else {
		return NULL;
	}
}

char *settings_attachment_dirname_get(void)
{
	if (load_settings()) {
		return settings_attachment_dirname;
	} else {
		return NULL;
	}
}

char *settings_imap_host_get(void)
{
	if (load_settings()) {
		return settings_imap_host;
	} else {
		return NULL;
	}
}

unsigned short settings_imap_port_get(void)
{
	if (load_settings()) {
		return settings_imap_port;
	} else {
		return 0;
	}
}

char *settings_imap_user_get(void)
{
	if (load_settings()) {
		return settings_imap_user;
	} else {
		return NULL;
	}
}

char *settings_imap_pass_get(void)
{
	if (load_settings()) {
		return settings_imap_pass;
	} else {
		return NULL;
	}
}

char *settings_smtp_host_get(void)
{
	if (load_settings()) {
		return settings_smtp_host;
	} else {
		return NULL;
	}
}

unsigned short settings_smtp_port_get(void)
{
	if (load_settings()) {
		return settings_smtp_port;
	} else {
		return 0;
	}
}

char *settings_smtp_user_get(void)
{
	if (load_settings()) {
		return settings_smtp_user;
	} else {
		return NULL;
	}
}

char *settings_smtp_pass_get(void)
{
	if (load_settings()) {
		return settings_smtp_pass;
	} else {
		return NULL;
	}
}
