/*
    QuickMSG
    Copyright (C) 2014  Jeroen Vreeken <jeroen@vreeken.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "smtp.h"
#include "message.h"
#include "settings.h"

#include <unistd.h>
#include <libetpan/libetpan.h>

static int send_message(char *data, size_t len, char *rcpt)
{
	int s = -1;
	int ret;
	int esmtp = 0;
	mailsmtp *smtp = NULL;
	int smtp_esmtp = 1;
	int smtp_tls = 1;

	char *smtp_server = settings_smtp_host_get();
	uint16_t smtp_port = settings_smtp_port_get();
	char *smtp_user = settings_smtp_user_get();
	char *smtp_password = settings_smtp_pass_get();
	char *smtp_from = settings_address_get();

	if ((smtp = mailsmtp_new(0, NULL)) == NULL) {
		perror("mailsmtp_new");
		goto error;
	}

	/* first open the stream */
	if ((ret = mailsmtp_socket_connect(smtp,
	    smtp_server,
	    smtp_port)) != MAILSMTP_NO_ERROR) {
		fprintf(stderr, "mailsmtp_socket_connect: %s\n", mailsmtp_strerror(ret));
		goto error;
	}
  
	/* then introduce ourselves */
	if (smtp_esmtp && (ret = mailesmtp_ehlo(smtp)) == MAILSMTP_NO_ERROR)
		esmtp = 1;
	else if (!smtp_esmtp || ret == MAILSMTP_ERROR_NOT_IMPLEMENTED)
		ret = mailsmtp_helo(smtp);

	if (ret != MAILSMTP_NO_ERROR) {
		fprintf(stderr, "mailsmtp_helo: %s\n", mailsmtp_strerror(ret));
		goto error;
	}

	if (esmtp && smtp_tls &&
	    (ret = mailsmtp_socket_starttls(smtp)) != MAILSMTP_NO_ERROR) {
		fprintf(stderr, "mailsmtp_starttls: %s\n", mailsmtp_strerror(ret));
//		goto error;
	}
  
	if (esmtp && smtp_tls) {
		/* introduce ourselves again */
		if (smtp_esmtp && (ret = mailesmtp_ehlo(smtp)) == MAILSMTP_NO_ERROR)
			esmtp = 1;
		else if (!smtp_esmtp || ret == MAILSMTP_ERROR_NOT_IMPLEMENTED)
			ret = mailsmtp_helo(smtp);

		if (ret != MAILSMTP_NO_ERROR) {
			fprintf(stderr, "mailsmtp_helo: %s\n", mailsmtp_strerror(ret));
			goto error;
		}
	}
  
	if (esmtp && smtp_user != NULL &&
	    (ret = mailsmtp_auth(smtp, smtp_user,
	    (smtp_password != NULL) ? smtp_password : ""))
	      != MAILSMTP_NO_ERROR) {
		fprintf(stderr, "mailsmtp_auth: %s: %s\n", smtp_user, mailsmtp_strerror(ret));
		goto error;
	}

	/* source */
	if ((ret = (esmtp ?
	    mailesmtp_mail(smtp, smtp_from, 1, "etPanSMTPTest") :
	    mailsmtp_mail(smtp, smtp_from))) != MAILSMTP_NO_ERROR) {
		fprintf(stderr, "mailsmtp_mail: %s, %s\n", smtp_from, mailsmtp_strerror(ret));
		goto error;
	}
  
	if ((ret = (esmtp ?
	    mailesmtp_rcpt(smtp, rcpt,
	    MAILSMTP_DSN_NOTIFY_FAILURE|MAILSMTP_DSN_NOTIFY_DELAY,
	    NULL) :
	    mailsmtp_rcpt(smtp, rcpt))) != MAILSMTP_NO_ERROR) {
		fprintf(stderr, "mailsmtp_rcpt: %s: %s\n", rcpt, mailsmtp_strerror(ret));
		goto error;
	}
  
	/* message */
	if ((ret = mailsmtp_data(smtp)) != MAILSMTP_NO_ERROR) {
		fprintf(stderr, "mailsmtp_data: %s\n", mailsmtp_strerror(ret));
		goto error;
	}

	if ((ret = mailsmtp_data_message(smtp, data, len)) != MAILSMTP_NO_ERROR) {
		fprintf(stderr, "mailsmtp_data_message: %s\n", mailsmtp_strerror(ret));
		goto error;
	}
	mailsmtp_free(smtp);
	return 0;

error:
	printf("Error sending mail\n");
	if (smtp != NULL)
		mailsmtp_free(smtp);
	if (s >= 0)
		close(s);
	return -1;
}



int smtp_send(char *to, struct attachment_list *al, char *type)
{
	int r;
	void *msg;
	size_t length;
	
	r = message_multipart_create(to, al, &msg, &length, type);

	printf("message_multipart_create: %d %p %zd\n", r, msg, length);

	send_message(msg, length, to);
	
	free(msg);



	return 0;
}

